###############################################################################
# Find xcf
#
# This sets the following variables:
# xcf_FOUND - True if OPENNI was found.
# XCF_INCLUDE_DIRS - Directories containing the OPENNI include files.
# XCF_LIBRARIES - Libraries needed to use OPENNI.
# XCF_DEFINITIONS - Compiler flags for OPENNI.
# 

find_package(PkgConfig)
 
if(${CMAKE_VERSION} VERSION_LESS 2.8.2)
  pkg_check_modules(PC_XCF xcf)
else()
  pkg_check_modules(PC_XCF QUIET xcf)
endif()

set(XCF_DEFINITIONS ${PC_XCF_CFLAGS_OTHER})

set(XCF_SUFFIX)
if(WIN32 AND CMAKE_SIZEOF_VOID_P EQUAL 8)
  set(XCF_SUFFIX 64)
endif(WIN32 AND CMAKE_SIZEOF_VOID_P EQUAL 8)

#add a hint so that it can find it without the pkg-config
find_path(XCF_INCLUDE_DIR xcf.hpp
          HINTS ${PC_XCF_INCLUDEDIR} ${PC_XCF_INCLUDE_DIRS} /usr/include/xcf "${XCF_ROOT}" "$ENV{XCF_ROOT}"
          PATHS "$ENV{OPEN_NI_INSTALL_PATH${OPENNI_SUFFIX}}/Include"
          PATH_SUFFIXES xcf include Include)
#add a hint so that it can find it without the pkg-config
find_library(XCF_LIBRARY 
             NAMES xcf${XCF_SUFFIX}
             HINTS ${PC_XCF_LIBDIR} ${PC_XCF_LIBRARY_DIRS} /usr/lib "${XCF_ROOT}" "$ENV{XCF_ROOT}"
             PATHS "$ENV{XCF_LIB${OPENNI_SUFFIX}}"
             PATH_SUFFIXES lib Lib Lib64)

set(XCF_INCLUDE_DIRS ${XCF_INCLUDE_DIR})
set(XCF_LIBRARIES ${XCF_LIBRARY})

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(xcf DEFAULT_MSG
    XCF_LIBRARY XCF_INCLUDE_DIR)
    
mark_as_advanced(XCF_LIBRARY XCF_INCLUDE_DIR)

if(XCF_FOUND)
  set(xcf_FOUND XCF_FOUND)
  # Add the include directories
  set(XCF_INCLUDE_DIRS ${XCF_INCLUDE_DIRS})
  include_directories(SYSTEM ${XCF_INCLUDE_DIRS})
  if(NOT xcf_FIND_QUIETLY)
    message(STATUS "xcf found (include: ${XCF_INCLUDE_DIRS}, lib: ${XCF_LIBRARY})")
  endif(NOT xcf_FIND_QUIETLY)
endif(XCF_FOUND)
