/*
 * Window.cpp
 *
 *  Created on: Jul 25, 2012
 *      Author: lziegler
 */

#include <QtGui>
#include <iostream>

#include "Window.h"

using namespace kgf;
using namespace std;

Window::Window(string grabberImpl, QWidget* parent) :
		QDialog(parent), doPaint(true) {
	depthLabel = new QLabel(this);
	rgbLabel = new QLabel(this);
	QLabel *grabberLabel = new QLabel(QString(grabberImpl.c_str()), this);
	paintCheck = new QCheckBox("paint", this);
	paintCheck->setChecked(doPaint);
	connect(paintCheck, SIGNAL(toggled(bool)), this, SLOT(togglePaint(bool)));

	depthLabel->setMinimumSize(640, 480);
	rgbLabel->setMinimumSize(640, 480);

	QGridLayout *grid = new QGridLayout;
	grid->addWidget(rgbLabel, 0, 0);
	grid->addWidget(depthLabel, 0, 1);
	grid->addWidget(grabberLabel, 1, 0);
	grid->addWidget(paintCheck, 2, 0);

	grid->setColumnStretch(0, 1);
	grid->setColumnStretch(1, 1);

	setLayout(grid);

	setWindowTitle(tr("Kinect Preview - ") + QString(grabberImpl.c_str()));
	resize(1280, 480);

}

QImage Window::paintRgb(Image<PixelRGB>::Ptr imgRgb) {

	if (imgRgb.get() != 0) {
		// Paint a RGB image
		QImage img((uchar*) imgRgb->data_ptr(), imgRgb->getWidth(),
				imgRgb->getHeight(), QImage::Format_RGB888);
		return img;
	}
	return QImage();
}

QImage Window::paintDepth(Image<PixelRawDepth>::Ptr imgDepth) {

	if (imgDepth.get() != 0) {
		// Paint a Depth image
		QImage img(imgDepth->getWidth(), imgDepth->getHeight(),
				QImage::Format_RGB888);
		for (uint row = 0; row < imgDepth->getHeight(); row++) {
			for (uint col = 0; col < imgDepth->getWidth(); col++) {
				int val = 256 - (imgDepth->at(col, row) / 20) % 256;
				img.setPixel(col, row, qRgb(val, val, val));
			}
		}
		return img;
	}
	return QImage();
}

void Window::paintEvent(QPaintEvent *event) {

	if (!doPaint || !newData)
		return;

	boost::mutex::scoped_lock lock(paintMutex);

	QImage img0 = paintRgb(imgRgb);
	QImage img1 = paintDepth(imgDepth);

	// paint the pixels where the empty labels are
	QPainter *painter = new QPainter(this);
	painter->drawImage(rgbLabel->pos(), img0);
	painter->drawImage(depthLabel->pos(), img1);

	newData = false;
}

void Window::setImages(Image<PixelRawDepth>::Ptr imgDepthIn,
		Image<PixelRGB>::Ptr imgRgbIn) {
	if (!doPaint && newData)
		return;

	// skip incoming frame if last frame is still being progressed
	if (paintMutex.try_lock()) {
		paintMutex.unlock();
		imgDepth = imgDepthIn;
		imgRgb = imgRgbIn;
		newData = true;
		update();
	}
}

Window::~Window() {

}
