/*
 * kgList.cpp
 *
 *  Created on: Jul 16, 2012
 *      Author: lziegler
 */

#include "Window.h"

#include <iostream>
#include <iomanip>
#include <vector>
#include <QtGui>
#include <boost/thread.hpp>
#include <log4cxx/basicconfigurator.h>

/*
 * KGF header inclusion
 */
#include <kgfconfig.h>
#include <KGF/GrabberUtils.h>

#ifdef KGF_HAVE_OPENNI
#include <KGF/OpenNIGrabber.h>
#include <KGF/OpenNIPlayback.h>
#endif
#ifdef KGF_HAVE_FREENECT
#include <KGF/FreenectGrabber.h>
#endif
#ifdef KGF_HAVE_XCF
#include <KGF/KinectServerGrabber.h>
#endif

using namespace std;
using namespace kgf;
using namespace boost;

int histsize = 100;
long lastTime = -1;
vector<long> hist;

void analyzeFps(long timestamp) {
	if (lastTime < 0) {
		lastTime = timestamp;
		return;
	}
	hist.push_back(timestamp - lastTime);
	lastTime = timestamp;
	if (hist.size() >= histsize) {
		long sum = 0;
		for (vector<long>::iterator it = hist.begin(); it < hist.end();
				++it) {
			sum += *it;
		}
		float diff = float(sum) / float(hist.size());
		float fps = 1000.0 / diff;
		cout << "Grabbing: " << setiosflags(ios::left) << setprecision(4)
				<< setw(5) << fps << " fps" << endl;
		hist.clear();
	}
}

void loop(Window *window, Grabber::Ptr grabber) {
	while (true) {
		try {
			// wait for new frames
			grabber->next();

			// get pointers to the data of the current frame
			Image<PixelRGB>::Ptr imgRgb = grabber->getLastImageRGB();
			Image<PixelRawDepth>::Ptr imgDepth = grabber->getLastImageDepth();

			// update gui
			window->setImages(imgDepth, imgRgb);

			analyzeFps(GrabberUtils::currentTimeMillis());

			this_thread::interruption_point();
		} catch (thread_interrupted &e) {
			break;
		}
	}
}

int main(int argc, char **argv) {

	log4cxx::BasicConfigurator::configure();
	log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getWarn());

	Grabber::Ptr grabber;
// Create desired grabber instance
#ifdef KGF_HAVE_OPENNI
	grabber = Grabber::Ptr(new OpenNIGrabber());
#else
#ifdef KGF_HAVE_FREENECT
	grabber = Grabber::Ptr(new FreenectGrabber());
#else
#ifdef KGF_HAVE_XCF
	grabber = Grabber::Ptr(new KinectServerGrabber());
#else
	cerr << "No grabber available" << endl;
	return 1;
#endif
#endif
#endif

	// Use default configuration
	GrabConfigurationRgb configRgb;
	GrabConfigurationDepth configDepth;

	// Start grabbing
	try {
		grabber->init(configRgb, configDepth);
	} catch (InitializationException &e) {
		cerr << "InitializationException: " << e.what() << endl;
		return 1;
	}

// Create Preview Window
	QApplication app(argc, argv);
	Window *window = new Window(grabber->getKey());
	window->show();

// Continuously update preview window by polling
	thread thread0(loop, window, grabber);

	int ret = app.exec();

	thread0.interrupt();
	thread0.join();

	return ret;
}
