/*
 * kgGrabAsynchronous.cpp
 *
 *  Created on: Oct 25, 2012
 *      Author: lziegler
 */

#include <KGF/kgf.h>

#include <log4cxx/basicconfigurator.h>

using namespace std;

class MyObserver: public kgf::GrabObserver {
public:

	virtual void processImageRGB(kgf::Image<kgf::PixelRGB>::Ptr) {
		cout << "got rgb image" << endl;
	}

	virtual void processImageDepth(kgf::Image<kgf::PixelRawDepth>::Ptr) {
		cout << "got depth image" << endl;
	}

	void processImageXYZ(kgf::Image<kgf::PixelXYZ>::Ptr) {
		cout << "got xyz image" << endl;
	}

	virtual ~MyObserver() {
	}
};

int main(int argc, char **argv) {

	log4cxx::BasicConfigurator::configure();
	log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getWarn());

	cout << "\nThis is kgGrabAsynchronous" << endl;
	cout << endl;

	{
		// auto select a grabber
		kgf::Grabber::Ptr grabber = kgf::autoSelectGrabber();

		// configure and initialize the grabber
		kgf::GrabConfigurationDepth configD(640, 480, 30);
		kgf::GrabConfigurationRgb configR(640, 480, 30);
		grabber->init(configR, configD);

		// register a observer
		kgf::GrabObserver::Ptr myObserver(new MyObserver());
		grabber->registerObserver(myObserver);

		// do something for 2 seconds
		long timeStartMilli = kgf::GrabberUtils::currentTimeMillis();
		while (kgf::GrabberUtils::currentTimeMillis() < timeStartMilli + 2000) {
			usleep(100000);
		}

		grabber->unregisterObserver(myObserver);

		cout << endl;
		cout << "Shutting down..." << endl;
	}
	// grabber should be shut down at this point

	cout << "Done" << endl;
}
