/*
 * Window.cpp
 *
 *  Created on: Jul 25, 2012
 *      Author: lziegler
 */

#include <QtGui>
#include <iostream>

#include "Window.h"

using namespace kgf;
using namespace std;

Window::Window(Controller::Ptr controller, QWidget *parent) :
		QMainWindow(parent), controller(controller) {
	QWidget *w = new QWidget(this);
	QGridLayout *grid = new QGridLayout;
	grid->addWidget(createDisplayGroup(), 0, 0);
	grid->addWidget(createStaticGroup(), 0, 1);
	grid->addWidget(createMovingGroup(), 0, 2);

	grid->setColumnStretch(0, 1);
	grid->setColumnStretch(1, 1);
	grid->setColumnStretch(2, 2);

	w->setLayout(grid);

	setCentralWidget(w);

	setWindowTitle(tr("Kinect Grabbing Gui"));
	resize(640, 320);
}

void Window::closeEvent(QCloseEvent *event) {
	if (previewWindow.get() != 0) {
		previewWindow->close();
	}
}

void Window::startStatic() {
	if (list->findItems(line->text(), Qt::MatchExactly).size() == 0) {
		list->addItem(line->text());
	}
	startButtonStatic->setEnabled(false);
	stopButtonStatic->setEnabled(true);
	progressBarStatic->setRange(0, 0);
	progressBarStatic->setFormat("Done");
	list->setEnabled(false);
	line->setEnabled(false);

	controller->startRecStatic(line->text().toStdString());
}
void Window::stopStatic() {
	startButtonStatic->setEnabled(true);
	stopButtonStatic->setEnabled(false);
	progressBarStatic->setRange(0, 1);
	progressBarStatic->setValue(1);
	progressBarStatic->setFormat("Done");
	list->setEnabled(true);
	line->setEnabled(true);

	controller->stopRecStatic();
}

void Window::check() {
	controller->setPositionStream(checkLine->text().toStdString());
	bool ok = controller->checkPositionStream();
	if (ok) {
		checkLabel->setText("ok");
	} else {
		checkLabel->setText("ERROR");
	}
}
void Window::display() {
	// create preview window
	if (previewWindow.get() == 0) {
		previewWindow = DisplayWindow::Ptr(new DisplayWindow(controller));
		controller->display(previewWindow);
		previewWindow->show();
		connect(previewWindow.get(), SIGNAL(rejected()), this,
				SLOT(closeDisplay()));
	} else {
		previewWindow->raise();
		previewWindow->activateWindow();
	}
}
void Window::closeDisplay() {
	// This is called right before object is destroyed
	controller->unregister(previewWindow);
	previewWindow.reset();
}
void Window::startMoving() {
	startButtonMoving->setEnabled(false);
	stopButtonMoving->setEnabled(true);
	progressBarMoving->setRange(0, 0);
	progressBarMoving->setFormat("Done");

	controller->setPositionStream(checkLine->text().toStdString());
	controller->startRecMoving();
	controller->startRecPosition();
}
void Window::stopMoving() {
	startButtonMoving->setEnabled(true);
	stopButtonMoving->setEnabled(false);
	progressBarMoving->setRange(0, 1);
	progressBarMoving->setValue(1);
	progressBarMoving->setFormat("Done");

	controller->stopRecPosition();
	controller->stopRecMoving();
}

void Window::submitItem(QListWidgetItem *item) {
	line->setText(item->text());
	startButtonStatic->setEnabled(true);
}
void Window::editItem(const QString &text) {
	list->clearSelection();
	if (text.size() > 0) {
		startButtonStatic->setEnabled(true);
	} else {
		startButtonStatic->setEnabled(false);
	}
}

QGroupBox *Window::createDisplayGroup() {
	QGroupBox *groupBox = new QGroupBox(tr("Display Scene"));

	QVBoxLayout *vbox = new QVBoxLayout;
	QPushButton *displayButton = new QPushButton(tr("Start &Display"));
	vbox->addWidget(displayButton);

	vbox->addStretch(1);
	groupBox->setLayout(vbox);

	connect(displayButton, SIGNAL(released()), this, SLOT(display()));

	return groupBox;
}

QGroupBox *Window::createStaticGroup() {
	QGroupBox *groupBox = new QGroupBox(tr("Record Static Scene"));

	QVBoxLayout *vbox = new QVBoxLayout;

	line = new QLineEdit;
	list = new QListWidget;
	list->addItem("furniture");
	list->addItem("object");
	list->addItem("room");
	list->addItem("action");
	list->setSelectionMode(QAbstractItemView::SingleSelection);

	vbox->addWidget(new QLabel(tr("Scene content type:")));
	vbox->addWidget(line);
	vbox->addWidget(list);

	startButtonStatic = new QPushButton(tr("&Start Recording"));
	stopButtonStatic = new QPushButton(tr("&Stop Recording"));
	progressBarStatic = new QProgressBar();

	startButtonStatic->setEnabled(false);
	stopButtonStatic->setEnabled(false);
	progressBarStatic->setRange(0, 1);
	progressBarStatic->reset();

	vbox->addWidget(startButtonStatic);
	vbox->addWidget(stopButtonStatic);
	vbox->addWidget(progressBarStatic);

	connect(startButtonStatic, SIGNAL(released()), this, SLOT(startStatic()));
	connect(stopButtonStatic, SIGNAL(released()), this, SLOT(stopStatic()));
	connect(list, SIGNAL(itemPressed(QListWidgetItem*)), this,
			SLOT(submitItem(QListWidgetItem*)));
	connect(line, SIGNAL(textEdited(QString)), this, SLOT(editItem(QString)));

	vbox->addStretch(1);
	groupBox->setLayout(vbox);

	return groupBox;
}

QGroupBox *Window::createMovingGroup() {
	QGroupBox *groupBox = new QGroupBox(tr("Record Moving Scene"));

	QVBoxLayout *vbox = new QVBoxLayout;

	vbox->addWidget(new QLabel(tr("PositionData Stream:")));

	QHBoxLayout *hbox = new QHBoxLayout;
	checkLine = new QLineEdit;
	checkButton = new QPushButton(tr("Check"));
	checkLabel = new QLabel("");
	checkLabel->setMinimumWidth(50);
	hbox->addWidget(checkLine);
	hbox->addWidget(checkButton);
	hbox->addWidget(checkLabel);
	hbox->setContentsMargins(0, 0, 0, 0);
	QWidget *checkWidget = new QWidget();
	checkWidget->setLayout(hbox);
	vbox->addWidget(checkWidget);

	startButtonMoving = new QPushButton(tr("&Start Recording"));
	stopButtonMoving = new QPushButton(tr("S&top Recording"));
	progressBarMoving = new QProgressBar();

	startButtonMoving->setEnabled(true);
	stopButtonMoving->setEnabled(false);
	progressBarMoving->setRange(0, 1);
	progressBarMoving->reset();

	vbox->addSpacing(20);
	vbox->addWidget(startButtonMoving);
	vbox->addWidget(stopButtonMoving);
	vbox->addWidget(progressBarMoving);

	connect(startButtonMoving, SIGNAL(released()), this, SLOT(startMoving()));
	connect(stopButtonMoving, SIGNAL(released()), this, SLOT(stopMoving()));
	connect(checkButton, SIGNAL(released()), this, SLOT(check()));

	vbox->addStretch(1);
	groupBox->setLayout(vbox);

	return groupBox;
}

DisplayWindow::DisplayWindow(Controller::Ptr controller, QWidget* parent) :
		QDialog(parent), controller(controller) {
	depthLabel = new QLabel(this);
	rgbLabel = new QLabel(this);

	QPixmap depthPmap(640, 480);
	QPixmap rgbPmap(640, 480);

	depthLabel->setPixmap(depthPmap);
	rgbLabel->setPixmap(rgbPmap);

	QGridLayout *grid = new QGridLayout;
	grid->addWidget(rgbLabel, 0, 0);
	grid->addWidget(depthLabel, 0, 1);

	grid->setColumnStretch(0, 1);
	grid->setColumnStretch(1, 1);

	setLayout(grid);

	setWindowTitle(tr("Kinect Preview"));
	resize(1280, 480);
}
void DisplayWindow::paintRgb(Image<PixelRGB>::Ptr img) {
	if (img.get() != 0) {
		// Paint a RGB image
		QImage imageRgb(img->getWidth(), img->getHeight(),
				QImage::Format_RGB32);
		for (uint row = 0; row < img->getHeight(); row++) {
			for (uint col = 0; col < img->getWidth(); col++) {
				PixelRGB p = img->at(col, row);
				imageRgb.setPixel(col, row, qRgb(p.nRed, p.nGreen, p.nBlue));
			}
		}
		rgbLabel->setPixmap(QPixmap::fromImage(imageRgb));
	}
}

void DisplayWindow::paintDepth(Image<PixelRawDepth>::Ptr img) {
	if (img.get() != 0) {
		// Paint a Depth image
		QImage imageDepth(img->getWidth(), img->getHeight(),
				QImage::Format_RGB32);
		for (uint row = 0; row < img->getHeight(); row++) {
			for (uint col = 0; col < img->getWidth(); col++) {
				int val = (img->at(col, row) / 10) % 256;
				imageDepth.setPixel(col, row, qRgb(val, val, val));
			}
		}
		depthLabel->setPixmap(QBitmap::fromImage(imageDepth));
	}
}

void DisplayWindow::paintEvent(QPaintEvent *event) {

	paintMutex.lock();
	Image<PixelRawDepth>::Ptr imgDepth = this->imgDepth;
	Image<PixelRGB>::Ptr imgRgb = this->imgRgb;
	paintMutex.unlock();

	paintRgb(imgRgb);
	paintDepth(imgDepth);
}

void DisplayWindow::processImageRGB(Image<PixelRGB>::Ptr img) {
	boost::mutex::scoped_lock lock(paintMutex);
	imgRgb = img;
	update();
}

void DisplayWindow::processImageDepth(Image<PixelRawDepth>::Ptr img) {
	boost::mutex::scoped_lock lock(paintMutex);
	imgDepth = img;
	update();
}

void DisplayWindow::processImageXYZ(Image<PixelXYZ>::Ptr img) {
}

DisplayWindow::~DisplayWindow() {

}
