/*
 * Window.h
 *
 *  Created on: Jul 25, 2012
 *      Author: lziegler
 */

#ifndef WINDOW_H
#define WINDOW_H

#include <QtGui>
#include <boost/thread.hpp>
#include "Controller.h"

class QGroupBox;

class DisplayWindow: public QDialog,
		public kgf::GrabObserver,
		public boost::enable_shared_from_this<DisplayWindow> {
Q_OBJECT
public:
	typedef boost::shared_ptr<DisplayWindow> Ptr;
	DisplayWindow(Controller::Ptr controller, QWidget *parent = 0);
	virtual void processImageRGB(kgf::Image<kgf::PixelRGB>::Ptr);
	virtual void processImageDepth(kgf::Image<kgf::PixelRawDepth>::Ptr);
	virtual void processImageXYZ(kgf::Image<kgf::PixelXYZ>::Ptr);
	virtual Ptr getThis() {
		return shared_from_this();
	}
	virtual ~DisplayWindow();

protected:
    void paintEvent(QPaintEvent *event);
    void paintDepth(kgf::Image<kgf::PixelRawDepth>::Ptr img);
    void paintRgb(kgf::Image<kgf::PixelRGB>::Ptr img);

private:
	QLabel *depthLabel;
	QLabel *rgbLabel;
	boost::mutex paintMutex;
	kgf::Image<kgf::PixelRGB>::Ptr imgRgb;
	kgf::Image<kgf::PixelRawDepth>::Ptr imgDepth;
	Controller::Ptr controller;
};

class Window: public QMainWindow {
Q_OBJECT

public:
	Window(Controller::Ptr controller, QWidget *parent = 0);

private:
	QGroupBox *createDisplayGroup();
	QGroupBox *createStaticGroup();
	QGroupBox *createMovingGroup();

	QPushButton *startButtonStatic;
	QPushButton *stopButtonStatic;
	QProgressBar *progressBarStatic;
	QPushButton *startButtonMoving;
	QPushButton *stopButtonMoving;
	QProgressBar *progressBarMoving;
	QLineEdit *line;
	QListWidget *list;

	QLineEdit *checkLine;
	QPushButton *checkButton;
	QLabel *checkLabel;

	Controller::Ptr controller;
	DisplayWindow::Ptr previewWindow;

protected:
	void closeEvent(QCloseEvent *event);

private slots:
	void startStatic();
	void stopStatic();
	void startMoving();
	void stopMoving();
	void check();
	void display();
	void closeDisplay();
	void submitItem(QListWidgetItem *item);
	void editItem(const QString &text);
};

#endif
