/*
 * GrabberUtils.h
 *
 *  Created on: Jul 17, 2012
 *      Author: lziegler
 */

#ifndef GRABBERUTILS_H_
#define GRABBERUTILS_H_

#include "GrabberTypes.h"
#include "Grabber.h"
#include "kgfconfig.h"

#include <sys/time.h>
#include <time.h>

#include <string>
#include <sstream>

#ifdef KGF_HAVE_PCL
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#include <pcl/point_cloud.h>
#include <pcl/point_types.h>
#endif

namespace kgf {

class GrabberUtils {
public:

	static inline unsigned long currentTimeMillis() {
		timeval tv;
		gettimeofday(&tv, NULL);

		// (second-value in milliseconds) + (microsecond value in milliseconds)
		return (((unsigned long) tv.tv_sec) * 1000l) + (tv.tv_usec / 1000l);
	}
	static inline std::string toString(GrabConfiguration config) {
		std::stringstream ss;
		ss << "[CONFIG w:" << config.resWidth << " h:" << config.resHeight
				<< " fps:" << config.fps << "]";
		return ss.str();
	}
	static inline std::string toString(
			std::vector<GrabConfigurationRgb> configs) {
		std::string str = "";
		for (std::vector<GrabConfigurationRgb>::iterator it = configs.begin();
				it != configs.end(); ++it) {
			str += GrabberUtils::toString(*it);
			str += "\n";
		}
		return str;
	}
	static inline std::string toString(
			std::vector<GrabConfigurationDepth> configs) {
		std::string str = "";
		for (std::vector<GrabConfigurationDepth>::iterator it = configs.begin();
				it != configs.end(); ++it) {
			str += GrabberUtils::toString(*it);
			str += "\n";
		}
		return str;
	}

#ifdef KGF_HAVE_PCL
	static pcl::PointCloud<pcl::PointXYZ>::Ptr toPclCloud(
			Image<PixelXYZ>::Ptr xyzimage) {

		int width = xyzimage->getWidth();
		int height = xyzimage->getHeight();

		// create structured, but empty cloud
		pcl::PointCloud<pcl::PointXYZ>::Ptr cloud = pcl::PointCloud<
				pcl::PointXYZ>::Ptr(
				new pcl::PointCloud<pcl::PointXYZ>(width, height));

		for (int col = 0; col < width; col++) {
			for (int row = 0; row < height; row++) {
				PixelXYZ p = xyzimage->at(col, row);
				// only add valid points
				if (p.x != .0 && p.y != .0 && p.z != .0) {
					cloud->at(col, row) = pcl::PointXYZ(p.x, p.y, p.z);
				}
			}
		}

		return cloud;
	}
#endif

};
}

#endif /* GRABBERUTILS_H_ */
