/*
 * OpenNIGrabber.h
 *
 *  Created on: Feb 1, 2011
 *      Author: lziegler
 */

#ifndef OPENNIPLAYBACK_H_
#define OPENNIPLAYBACK_H_

#pragma GCC diagnostic ignored "-Wreorder"
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#pragma GCC diagnostic ignored "-Wunused-variable"
#pragma GCC diagnostic ignored "-Wreturn-type"
#include <XnOpenNI.h>
#include <XnCodecIDs.h>
#include <XnCppWrapper.h>
#include <vector>
#include <boost/thread.hpp>
#include <boost/thread/condition_variable.hpp>
#include <log4cxx/logger.h>

#include "OpenNIGrabber.h"

namespace kgf {

class OpenNIPlayback: public OpenNIGrabber {
private:
	std::string path;
	xn::Player player;
public:
	/**
	 * Constructs a new grabber instance.
	 */
	OpenNIPlayback(std::string path) :
			OpenNIGrabber(), path(path) {
		logger = log4cxx::Logger::getLogger("OpenNIPlayback");
		LOG4CXX_INFO(logger, "Creating OpenNIGrabber");
	}

	void init(GrabConfigurationRgb configRgb,
			GrabConfigurationDepth configDepth) throw (InitializationException) {

		LOG4CXX_INFO(logger, "Initializing OpenNILogger");
		this->configRgb = configRgb;
		this->configDepth = configDepth;

		// init OpenNI
		XnStatus nRetVal = XN_STATUS_OK;

		// Initialize context object
		nRetVal = context.Init();
		checkStatus(nRetVal, "init");

		nRetVal = context.OpenFileRecording(path.c_str(), player);
		checkStatus(nRetVal, std::string("opening file ") + path);
		player.SetPlaybackSpeed(1.0);

		xn::NodeInfoList nodes;
		nRetVal = player.EnumerateNodes(nodes);
		checkStatus(nRetVal, "EnumerateNodes");

		// find nodes
		bool foundDepth = false;
		bool foundRgb = false;
		for (xn::NodeInfoList::Iterator it = nodes.Begin(); it != nodes.End();
				++it) {
			xn::NodeInfo nodeInfo = *it;
			if (nodeInfo.GetDescription().Type == XN_NODE_TYPE_DEPTH) {
				nRetVal = nodeInfo.GetInstance(depthGen);
				checkStatus(nRetVal, "GetInstance(depthGen)");
				foundDepth = true;
			} else if (nodeInfo.GetDescription().Type == XN_NODE_TYPE_IMAGE) {
				nRetVal = nodeInfo.GetInstance(imageGen);
				checkStatus(nRetVal, "GetInstance(imageGen)");
				foundRgb = true;
			}
		}
		if (!foundDepth) {
			LOG4CXX_ERROR(logger, "Recording does not contain depth node!");
			throw InitializationException(
					"Recording does not contain depth node!");
		}
		if (!foundRgb) {
			LOG4CXX_ERROR(logger, "Recording does not contain image node!");
			throw InitializationException(
					"Recording does not contain image node!");
		}

		// TODO check if resolution and fps match

		// Make it start generating data
		nRetVal = context.StartGeneratingAll();
		checkStatus(nRetVal, "start generating");

		initialized = true;

		// start the threads, that actually grab from the generator nodes
		grabAllThread = boost::thread(&OpenNIGrabber::grabAllLoop, this);
		LOG4CXX_INFO(logger, "grabbing started");
	}

	virtual ~OpenNIPlayback() {
		LOG4CXX_INFO(logger, "Shutting down OpenNI");

		grabAllThread.interrupt();
		boost::posix_time::ptime deadline =
				boost::posix_time::microsec_clock::local_time()
						+ boost::posix_time::seconds(1);
		grabAllThread.timed_join(deadline);

		setEnabled(false);
		context.Release();
	}
};
}
#endif /* OPENNIPLAYBACK_H_ */
