/*
 * KGF.h
 *
 *  Created on: Oct 25, 2012
 *      Author: lziegler
 */

#ifndef KGF_H_
#define KGF_H_

#include <KGF/Grabber.h>
#include <KGF/GrabberUtils.h>
#include <kgfconfig.h>

#include <iostream>
#include <list>
#include <string>
#include <log4cxx/basicconfigurator.h>

#ifdef KGF_HAVE_OPENNI
#include <KGF/OpenNIGrabber.h>
#include <KGF/OpenNIPlayback.h>
#endif
#ifdef KGF_HAVE_FREENECT
#include <KGF/FreenectGrabber.h>
#endif
#ifdef KGF_HAVE_XCF
#include <KGF/KinectServerGrabber.h>
#endif

bool endsWith(std::string const &fullString, std::string const &ending) {
	if (fullString.length() >= ending.length()) {
		return (0
				== fullString.compare(fullString.length() - ending.length(),
						ending.length(), ending));
	} else {
		return false;
	}
}

namespace kgf {

/**
 * Automatically selects an available driver and returns the corresponding grabber object.
 * @return an automatically selected grabber instance
 */
Grabber::Ptr autoSelectGrabber() throw (GrabberException) {
#ifdef KGF_HAVE_OPENNI
	kgf::Grabber::Ptr grabo = kgf::OpenNIGrabber::Ptr(new kgf::OpenNIGrabber());
	return grabo;
#endif
#ifdef KGF_HAVE_FREENECT
	kgf::Grabber::Ptr grabf = kgf::FreenectGrabber::Ptr(
			new kgf::FreenectGrabber());
	return grabf;
#endif
#ifdef KGF_HAVE_XCF
	kgf::Grabber::Ptr grabx = kgf::KinectServerGrabber::Ptr(new kgf::KinectServerGrabber());
	return grabx;
#endif

	throw GrabberException(
			"No applicable driver found for auto-selecting grabber");
}

/**
 * Automatically selects an available driver and returns the corresponding playback object.
 * @return an automatically selected playback instance
 */
Grabber::Ptr autoSelectPlayback(const std::string &file)
		throw (GrabberException) {

	if (!endsWith(file, "oni")) {
		throw GrabberException("Unrecognized file type of file: " + file);
	}

#ifdef KGF_HAVE_OPENNI
	kgf::Grabber::Ptr grabo = kgf::OpenNIGrabber::Ptr(
			new kgf::OpenNIPlayback(file));
	return grabo;
#endif

	throw GrabberException(
			"No applicable driver found for auto-selecting playback object");
}

/**
 * Generates a list of all available grabbers and playback objects based on the available drivers.
 * @return A list containing the identifiers of the available implementations.
 */
std::list<std::string> listAvailableGrabbers() {

	std::list<std::string> list;
#ifdef KGF_HAVE_OPENNI
	kgf::Grabber::Ptr grabo = kgf::OpenNIGrabber::Ptr(new kgf::OpenNIGrabber());
	kgf::Grabber::Ptr grabp = kgf::OpenNIPlayback::Ptr(
			new kgf::OpenNIPlayback(""));
	list.push_back(grabo->getKey());
	list.push_back(grabp->getKey());
#endif
#ifdef KGF_HAVE_FREENECT
	kgf::Grabber::Ptr grabf = kgf::FreenectGrabber::Ptr(new kgf::FreenectGrabber());
	list.push_back(grabf->getKey());
#endif
#ifdef KGF_HAVE_XCF
	kgf::Grabber::Ptr grabx = kgf::KinectServerGrabber::Ptr(new kgf::KinectServerGrabber());
	list.push_back(grabx->getKey());
#endif
	return list;
}

}

#endif /* KGF_H_ */
