/*
 * Grabber.h
 *
 *  Created on: Jul 16, 2012
 *      Author: lziegler
 */

#ifndef DUMMYGRABBER_H_
#define DUMMYGRABBER_H_

#include <KGF/Grabber.h>
#include <KGF/GrabberUtils.h>
#include <vector>
#include <boost/thread.hpp>

namespace kgf {

class DummyGrabber: public Grabber {
public:
	typedef boost::shared_ptr<DummyGrabber> Ptr;

	virtual void init() {
		thread = boost::thread(&DummyGrabber::simGrab, this);
	}
	virtual void next() {
		usleep(1000 * 1000);
	}

	virtual void setEnabled(bool enabled) {

	}
	virtual std::string getKey() {
		return "DummyGrabber";
	}

	virtual Image<PixelRGB>::Ptr getLastImageRGB() {
		boost::mutex::scoped_lock lock(mutex);
		return imgRgb;
	}
	virtual Image<PixelRawDepth>::Ptr getLastImageRawDepth() {
		boost::mutex::scoped_lock lock(mutex);
		return imgDepth;
	}
	virtual Image<PixelRealDepth>::Ptr getLastImageRealDepth() {
		boost::mutex::scoped_lock lock(mutex);
		Image<PixelRealDepth>::Ptr img(
				new Image<PixelRealDepth>(640, 480,
						GrabberUtils::currentTimeMillis()));
		return img;
	}
	virtual Image<PixelXYZ>::Ptr getLastImageXYZ() {
		boost::mutex::scoped_lock lock(mutex);
		Image<PixelXYZ>::Ptr img(
				new Image<PixelXYZ>(640, 480,
						GrabberUtils::currentTimeMillis()));
		PixelXYZ p;
		p.x = 1;
		p.y = 1;
		p.z = 1;

		img->at(0) = p;
		return img;
	}

	virtual bool canGetTilt() {
		return false;
	}
	virtual double getTiltDegrees() {
		return .0;
	}

	virtual void registerObserver(GrabObserver::Ptr observer) {
		observers.push_back(observer);
	}
	virtual void unregisterObserver(GrabObserver::Ptr observer) {
		std::vector<GrabObserver::Ptr>::iterator item = std::find(
				observers.begin(), observers.end(), observer);
		if (item != observers.end()) {
			observers.erase(item);
		}
	}

	DummyGrabber() {
	}
	virtual ~DummyGrabber() {
	}

private:
	boost::thread thread;
	boost::mutex mutex;
	std::vector<GrabObserver::Ptr> observers;
	Image<PixelRGB>::Ptr imgRgb;
	Image<PixelRawDepth>::Ptr imgDepth;

	void simGrab() {
		bool switcher = true;
		while (true) {
			{
				boost::mutex::scoped_lock lock(mutex);
				imgRgb = Image<PixelRGB>::Ptr(
						new Image<PixelRGB>(640, 480,
								GrabberUtils::currentTimeMillis()));
				imgDepth = Image<PixelRawDepth>::Ptr(
						new Image<PixelRawDepth>(640, 480,
								GrabberUtils::currentTimeMillis()));
				switcher = !switcher;
				for (uint row = 0; row < 480; row++) {
					for (uint col = 0; col < 640; col++) {
						if (row / 10 % 2 == (int) switcher) {
							PixelRGB p;
							p.nBlue = 0;
							p.nRed = 0;
							p.nGreen = 0;
							imgRgb->at(col, row) = p;
							imgDepth->at(col, row) = 32767;
						} else {
							PixelRGB p;
							p.nBlue = 255;
							p.nRed = 255;
							p.nGreen = 255;
							imgRgb->at(col, row) = p;
							imgDepth->at(col, row) = 0;
						}
					}
				}
			}

			for (std::vector<GrabObserver::Ptr>::iterator o = observers.begin();
					o != observers.end(); ++o) {
				GrabObserver::Ptr obs = *o;
				obs->processImageRGB(imgRgb);
				obs->processImageRawDepth(imgDepth);
			}

			usleep(1000 * 1000);
		}
	}
};

}
#endif /* GRABBER_H_ */
