/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "ArticulatedScene.h"

namespace ASM {

ArticulatedScene::ArticulatedScene(int width, int height) {
	staticX = cv::Mat::zeros(height, width, CV_32F);
	staticY = cv::Mat::zeros(height, width, CV_32F);
	staticZ = cv::Mat::zeros(height, width, CV_32F);

	weights = cv::Mat::zeros(height, width, CV_32F);

	dynamicX = cv::Mat::zeros(height, width, CV_32F);
	dynamicY = cv::Mat::zeros(height, width, CV_32F);
	dynamicZ = cv::Mat::zeros(height, width, CV_32F);
}

ArticulatedScene::ArticulatedScene(const ArticulatedScene &other) {
	staticX = cv::Mat(other.staticX);
	staticY = cv::Mat(other.staticY);
	staticZ = cv::Mat(other.staticZ);

	weights = cv::Mat(other.weights);

	dynamicX = cv::Mat(other.dynamicX);
	dynamicY = cv::Mat(other.dynamicY);
	dynamicZ = cv::Mat(other.dynamicZ);
}
ArticulatedScene::ArticulatedScene(const ArticulatedScene::Ptr &other) {
	staticX = cv::Mat(other->staticX);
	staticY = cv::Mat(other->staticY);
	staticZ = cv::Mat(other->staticZ);

	weights = cv::Mat(other->weights);

	dynamicX = cv::Mat(other->dynamicX);
	dynamicY = cv::Mat(other->dynamicY);
	dynamicZ = cv::Mat(other->dynamicZ);
}

ArticulatedScene::~ArticulatedScene() {
}

} /* namespace ASM */
