/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#ifndef ARTICULATEDSCENE_H_
#define ARTICULATEDSCENE_H_

#include <opencv2/core/core.hpp>
#include <boost/shared_ptr.hpp>

namespace ASM {

/**
 * A data class holding all relevant parts of the scene model.
 */
class ArticulatedScene {
public:
	typedef boost::shared_ptr<ArticulatedScene> Ptr;

	ArticulatedScene(int width, int height);
	ArticulatedScene(const ArticulatedScene &other);
	ArticulatedScene(const ArticulatedScene::Ptr &other);
	virtual ~ArticulatedScene();

	cv::Mat staticX;
	cv::Mat staticY;
	cv::Mat staticZ;

	cv::Mat weights;

	cv::Mat dynamicX;
	cv::Mat dynamicY;
	cv::Mat dynamicZ;
};

} /* namespace ASM */
#endif /* ARTICULATEDSCENE_H_ */
