/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "XyzImageData.h"

namespace ASM {

XyzImageData::XyzImageData() :
		_width(0), _height(0) {
}
XyzImageData::XyzImageData(int width, int height) :
		dataX(width * height), dataY(width * height), dataZ(width * height), _height(
				height), _width(width) {
}
XyzImageData::XyzImageData(const XyzImageData &other) {
	dataX = std::vector<float>(other.dataX);
	dataY = std::vector<float>(other.dataY);
	dataZ = std::vector<float>(other.dataZ);
	_height = other._height;
	_width = other._width;
}
XyzImageData::~XyzImageData() {
}

int XyzImageData::width() const {
	return _width;
}

int XyzImageData::height() const {
	return _height;
}
void XyzImageData::setWidth(int value) {
	_width = value;
}
void XyzImageData::setHeight(int value) {
	_height = value;
}
float XyzImageData::x(int index) const {
	return dataX[index];
}
float XyzImageData::y(int index) const {
	return dataY[index];
}
float XyzImageData::z(int index) const {
	return dataZ[index];
}
void XyzImageData::addX(float value) {
	dataX.push_back(value);
}
void XyzImageData::addY(float value) {
	dataY.push_back(value);
}
void XyzImageData::addZ(float value) {
	dataZ.push_back(value);
}
float* XyzImageData::mutableDataX() {
	return dataX.data();
}
float* XyzImageData::mutableDataY() {
	return dataY.data();
}
float* XyzImageData::mutableDataZ() {
	return dataZ.data();
}

} /* namespace ASM */
