/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#ifndef XYZIMAGEDATA_H_
#define XYZIMAGEDATA_H_

#include <boost/shared_ptr.hpp>
#include <vector>

namespace ASM {

/**
 * A base class for xyz image data.
 */
class XyzImageData {
protected:
	std::vector<float> dataX;
	std::vector<float> dataY;
	std::vector<float> dataZ;
	int _width;
	int _height;
public:
	typedef boost::shared_ptr<XyzImageData> Ptr;

	XyzImageData();
	XyzImageData(int width, int height);
	XyzImageData(const XyzImageData &other);
	virtual ~XyzImageData();

	virtual int width() const;
	virtual int height() const;

	virtual void setWidth(int value);
	virtual void setHeight(int value);

	virtual float x(int index) const;
	virtual float y(int index) const;
	virtual float z(int index) const;

	virtual void addX(float value);
	virtual void addY(float value);
	virtual void addZ(float value);

	virtual float* mutableDataX();
	virtual float* mutableDataY();
	virtual float* mutableDataZ();
};

} /* namespace ASM */
#endif /* XYZIMAGEDATA_H_ */
