/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "RsbConnectorIn.h"

#include <rsb/QueuePushHandler.h>
#include <rsb/Factory.h>
#include <rsb/Listener.h>
#include <iostream>

using namespace rsb;
using namespace rst::vision;
using namespace std;

namespace ASM {

RsbConnectorIn::RsbConnectorIn(const rsb::Scope &scope) {
	listener = rsb::Factory::getInstance().createListener(scope);

	imageQueue.reset(new rsc::threading::SynchronizedQueue<RstImagePtr>(1));

	listener->addHandler(
			rsb::HandlerPtr(
					new rsb::QueuePushHandler<SimpleXYZImage>(imageQueue)));
}

RsbConnectorIn::~RsbConnectorIn() {
}

XyzImageData::Ptr RsbConnectorIn::get() {

	RstImagePtr image = imageQueue->pop();
	RstXyzImageData::Ptr data(new RstXyzImageData(image));

	return data;
}

} /* namespace ASM */
