/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#ifndef RSBSERVER_H_
#define RSBSERVER_H_

#include "../Server.h"
#include "RstXyzImageData.h"

#include <rsb/Factory.h>
#include <rsb/patterns/Server.h>
#include <rst/vision/SimpleXYZImage.pb.h>

namespace ASM {

typedef boost::shared_ptr<rst::vision::SimpleXYZImage> ImagePtr;

class RsbServer: public ASM::Server {
public:
	RsbServer(const rsb::Scope &scope);
	virtual ~RsbServer();

	virtual void setSceneComputer(ArticulatedSceneComputer &comp);

private:
	rsb::patterns::ServerPtr server;

	class InitCallback: public rsb::patterns::Server::Callback<
			rst::vision::SimpleXYZImage, std::string> {
	public:
		ArticulatedSceneComputer &comp;
		InitCallback(ArticulatedSceneComputer &comp) :
				comp(comp) {
		}

		boost::shared_ptr<std::string> call(const std::string& /*methodName*/,
				ImagePtr initScene) {
			XyzImageData::Ptr image(new RstXyzImageData(initScene));
			comp.initialize(image);
			return boost::shared_ptr<std::string>(new std::string("init done"));
		}
	};
	class ResetCallback: public rsb::patterns::Server::Callback<void, std::string> {
	public:
		ArticulatedSceneComputer &comp;
		ResetCallback(ArticulatedSceneComputer &comp) :
				comp(comp) {
		}

		boost::shared_ptr<std::string> call(const std::string& /*methodName*/) {
			comp.reset();
			return boost::shared_ptr<std::string>(new std::string("reset done."));
		}
	};
};

} /* namespace ASM */
#endif /* RSBSERVER_H_ */
