/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#ifndef RSTSIMPLEXYZDATA_H_
#define RSTSIMPLEXYZDATA_H_

#include <asm/XyzImageData.h>

#include <rst/vision/SimpleXYZImage.pb.h>

typedef boost::shared_ptr<rst::vision::SimpleXYZImage> RstImagePtr;

namespace ASM {

class RstXyzImageData: public XyzImageData {
public:
	typedef boost::shared_ptr<RstXyzImageData> Ptr;

	RstXyzImageData(RstImagePtr img) :
			img(img) {
	}
	virtual ~RstXyzImageData() {
	}

	virtual inline int width() const {
		return img->width();
	}

	virtual inline int height() const {
		return img->height();
	}
	virtual inline void setWidth(int value) {
		img->set_width(value);
	}
	virtual inline void setHeight(int value) {
		img->set_height(value);
	}
	virtual inline float x(int index) const {
		return img->x(index);
	}
	virtual inline float y(int index) const {
		return img->y(index);
	}
	virtual inline float z(int index) const {
		return img->z(index);
	}
	virtual inline void addX(float value) {
		img->add_x(value);
	}
	virtual inline void addY(float value) {
		img->add_y(value);
	}
	virtual inline void addZ(float value) {
		img->add_z(value);
	}
	virtual inline float* mutableDataX() {
		return img->mutable_x()->mutable_data();
	}
	virtual inline float* mutableDataY() {
		return img->mutable_y()->mutable_data();
	}
	virtual inline float* mutableDataZ() {
		return img->mutable_z()->mutable_data();
	}
	inline RstImagePtr getRstPtr() {
		return img;
	}
private:
	RstImagePtr img;
};

} /* namespace ASM */
#endif /* RSTSIMPLEXYZDATA_H_ */
