/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "ComputeArticulatedScene.h"
#include <boost/bind.hpp>

namespace ASM {

ComputeArticulatedScene::ComputeArticulatedScene(
		ArticulatedSceneComputer &asmComp) :
		asmComp(asmComp), run(true) {
}

void ComputeArticulatedScene::initCommunication(ConnectorIn::Ptr in,
		ConnectorOut::Ptr outStatic, ConnectorOut::Ptr outDynamic,
		Server::Ptr server) {
	this->in = in;
	this->outStatic = outStatic;
	this->outDynamic = outDynamic;
	this->server = server;
	this->server->setSceneComputer(asmComp);
	this->worker = boost::thread(boost::bind(&ComputeArticulatedScene::work, this));
}

void ComputeArticulatedScene::work() {
	while (run) {
		// grab current image
		XyzImageData::Ptr image = in->get();

		// do work
		asmComp.work(image);

		// publish current model
		outStatic->put(asmComp.getCurrentStatic());
		outDynamic->put(asmComp.getCurrentDynamic());
	}
}

ComputeArticulatedScene::~ComputeArticulatedScene() {
	run = false;
	worker.join();
}

} /* namespace ASM */
