/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#ifndef COMPUTEARTICULATEDSCENE_H_
#define COMPUTEARTICULATEDSCENE_H_

#include "../comm/ConnectorIn.h"
#include "../comm/ConnectorOut.h"
#include "../comm/Server.h"
#include <asm/ArticulatedSceneComputer.h>
#include <boost/thread.hpp>

namespace ASM {

class ComputeArticulatedScene {
public:
	ComputeArticulatedScene(ArticulatedSceneComputer &asmComp);

	void initCommunication(ConnectorIn::Ptr in, ConnectorOut::Ptr outStatic,
			ConnectorOut::Ptr outDynamic, Server::Ptr server);

	virtual ~ComputeArticulatedScene();
protected:
	ArticulatedSceneComputer &asmComp;
	ConnectorIn::Ptr in;
	ConnectorOut::Ptr outStatic, outDynamic;
	Server::Ptr server;
	boost::thread worker;
	void work();
	bool run;
};

} /* namespace ASM */
#endif /* COMPUTEARTICULATEDSCENE_H_ */
