/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "GrabXyzImage.h"

#include <opencv/cv.h>
#include <boost/bind.hpp>

/*
 * KGF header inclusion
 */
#include <kgfconfig.h>
#include <KGF/kgf.h>
#include <KGF/GrabberUtils.h>

using namespace cv;

namespace ASM {

GrabXyzImage::GrabXyzImage(const ConnectorOut::Ptr connector, int width,
		int height, int fps) :
		connector(connector), width(width), height(height), fps(fps) {

	terminate = false;
	thread = boost::thread(boost::bind(&GrabXyzImage::work, this));
}

GrabXyzImage::~GrabXyzImage() {
	terminate = true;
	thread.join();
}

void GrabXyzImage::work() {
	kgf::Grabber::Ptr grabber = kgf::autoSelectGrabber();

	// Use default configuration
	kgf::GrabConfigurationDepth configDepth(width, height, fps);
	kgf::GrabConfigurationRgb configRgb(width, height, fps);

	// Start grabbing
	try {
		grabber->init(configRgb, configDepth);
		grabber->setXyzEnabled(true);
	} catch (kgf::InitializationException &e) {
		std::cerr << "InitializationException: " << e.what() << std::endl;
		return;
	}

	while (!terminate) {
		//grab the next image
		grabber->next();

		kgf::Image<kgf::PixelRGB>::Ptr imgRgb = grabber->getLastImageRGB();
		kgf::Image<kgf::PixelXYZ>::Ptr imgXYZ = grabber->getLastImageXYZ();

		// Create and publish an instance of SimpleImage. To see the
		// event, you can, for example use the RSB logger utility or the
		// receiver program in this directory.
		//if(imgXYZ!=0){
		XyzImageData::Ptr data(new XyzImageData());
		data->setWidth(width);
		data->setHeight(height);

		if (imgXYZ == 0) {
			continue;
			std::cerr << "Error grabbing data" << std::endl;
		}

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				kgf::PixelXYZ point = imgXYZ->at(x, y);
				//kgf publishes its data in millimeters, needs to be converted
				//to meters when imported into RSB structure
				data->addX(point.x);
				data->addY(point.y);
				data->addZ(point.z);

			}
		}
		connector->put(data);
	}
}
}
