/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include "VisualizeXyzImage.h"

#include <opencv2/highgui/highgui.hpp>
#include <opencv2/core/core.hpp>

#include <rsc/misc/langutils.h>

using namespace std;
using namespace cv;
using namespace boost;

namespace ASM {

void VisualizeXyzImage::analyzeFps(long timestamp) {
	if (lastTime < 0) {
		lastTime = timestamp;
		return;
	}
	hist.push_back(timestamp - lastTime);
	lastTime = timestamp;
	if (hist.size() >= histsize) {
		long sum = 0;
		for (vector<long>::iterator it = hist.begin(); it < hist.end();
				++it) {
			sum += *it;
		}
		float diff = float(sum) / float(hist.size());
		float fps = 1000.0 / diff;
		cout << "Visualizing " << windowname << ": " << setiosflags(ios::left) << setprecision(4)
				<< setw(5) << fps << " fps" << endl;
		hist.clear();
	}
}

VisualizeXyzImage::VisualizeXyzImage(const ConnectorIn::Ptr conIn,
		const std::string& windowname) :
		connector(conIn), windowname(windowname), run(true) {
	worker = thread(bind(&VisualizeXyzImage::work, this));
	lastTime = -1;
}

VisualizeXyzImage::~VisualizeXyzImage() {
	run = false;
	worker.join();
	cv::destroyWindow(windowname);
}

void VisualizeXyzImage::work() {
	while (run) {
		XyzImageData::Ptr image = connector->get();
		float *a = (float *) image->mutableDataZ();

		Mat aMat(image->height(), image->width(), CV_32F, a);

		// linearly lower pixel values for visualization
		aMat /= 4;

		analyzeFps(rsc::misc::currentTimeMillis());

		imshow(windowname, aMat);
	}
}

}
