/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include <boost/thread/thread.hpp>
#include <asm_nodes/nodes/GrabXyzImage.h>
#include <asm_nodes/nodes/VisualizeXyzImage.h>
#include <asm_nodes/comm/rsb/RsbConnectorIn.h>
#include <asm_nodes/comm/rsb/RsbConnectorOut.h>
#include <rsb/Factory.h>
#include <rsb/converter/ProtocolBufferConverter.h>
#include <rsb/converter/Repository.h>
#include <rst/vision/SimpleXYZImage.pb.h>
#include <log4cxx/basicconfigurator.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/core/core.hpp>
#include <iostream>

#include "tools/info.h"

using namespace rsb;
using namespace rsb::converter;
using namespace rst::vision;

using namespace std;

using namespace ASM;

int main(int argc, char **argv) {

	cout << ASM::infoText() << endl;

	log4cxx::BasicConfigurator::configure();
	log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getWarn());

	// Register a specific template instantiation of the
	// ProtocolBufferConverter for our SimpleXYZImage protocol buffer
	// message.
	boost::shared_ptr<ProtocolBufferConverter<SimpleXYZImage> > converter(
			new ProtocolBufferConverter<SimpleXYZImage>());
	converterRepository<std::string>()->registerConverter(converter);

	// Open a Window and register it to the scope on which we publish
	// raw depth images from the kinect
	cv::namedWindow("depth", CV_WINDOW_AUTOSIZE);
	VisualizeXyzImage vizNode(
			ConnectorIn::Ptr(new RsbConnectorIn(Scope("/bla/raw"))), "depth");

	// Grabbing
	GrabXyzImage grabNode(
			ConnectorOut::Ptr(new RsbConnectorOut(Scope("/bla/raw"))), 640, 480,
			30);

	std::cout << " done" << std::endl;
	std::cout << "press ESC to quit" << std::endl;

	while (true) {
		char k = cvWaitKey(5);
		if (k == 27) {
			cvDestroyWindow("rgb");
			cvDestroyWindow("depth");
			break;
		}
	}

	return 0;
}
