/*
* This file is part of Articulated Scene Model.
*
* Copyright(c) Agnes Swadzba, Leon Ziegler
* http://opensource.cit-ec.de/projects/asm
*
* This file may be licensed under the terms of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*/

#include <stdlib.h>

#include <rsb/Factory.h>
#include <iostream>
#include "tools/info.h"

using namespace rsb;
using namespace rsb::patterns;
using namespace std;

int main ( int argc, char **argv )
{
	cout << ASM::infoText() << endl;

    // Use the RSB factory to create a RemoteServer instance for the
    // server at scope /example/server.
    RemoteServerPtr remoteServer
        = rsb::Factory::getInstance().createRemoteServer("/bla/server");
        
    char input;
    while(true){
    
    	std::cout << "Reset Articulated Scene Model? [y/n]" << std::endl;
    	std::cin >> input;
    	
    	if(input == 'y'){

    		// Call the method "echo", passing it a string value as argument
    		// and accepting a string value as result. Note that the types of
    		// arguments and return values are defined by the server providing
    		// the respective methods and have to be matched in method calls.
    		// boost::shared_ptr<std::string> request(new std::string("reset"));
    		boost::shared_ptr<std::string> result
        		= remoteServer->call<std::string>("ASMreset");
    		std::cout << "Server replied: " << *result << std::endl;
    		
    	}
    	
    }

    return EXIT_SUCCESS;
    
}

